MultiSelection = function(length) {
	this.length = length
	this.listeners = new Array()
	this.selections = new Array()
	this.anchor = -1
	this.multiEnabled = true
	this.editEnabled = false
	this._editIndex = -1
	this._editLocks = new Array()
}

// Publics
MultiSelection.prototype.reset = function() {
	this.selections = new Array()
	this._editIndex = -1
	this.anchor = -1
}
MultiSelection.prototype.addEditLock = function(row) {
	this._editLocks.push(row)
}
MultiSelection.prototype.clickAgain = function() {
	this.click(this._lastClickedRow, 0, false, false)
}
MultiSelection.prototype.deselectAll = function(from, to) {
	if(this.editEnabled && this._editIndex != -1) {
		this.fireOnEndEdit(this._editIndex);
		this._editIndex = -1;
	}

	if(typeof from == "undefined") from = 0
	if(typeof to == "undefined") to = this.selections.length
	
	for(var i = this.selections.length - 1; i >= 0; i--) {
		this._deselectAt(i)
	}
}
MultiSelection.prototype.selectAll = function() {
	this.deselectAll()
	
	for(var i = 0; i < this.length; i++) {
		this._select(i, 0)
	}
}
MultiSelection.prototype.isSelected = function(row) {
	for(var i = 0; i < this.selections.length; i++) {
		if(this.selections[i] == row) return true
	}
	return false
}
MultiSelection.prototype.isSelectedAll = function() {
	return this.selections.length == this.length
}
MultiSelection.prototype.click = function(row, col, ctrl, shift) {
	this._lastClickedRow = row
	var cancel = this.fireOnPreClick(row, col, ctrl, shift)
	if(cancel) {
		return
	}
	
	if(this.editEnabled && this._editIndex != -1) {
		if(this._endEdit(this._editIndex)) {
			return
		}
		
		this._editIndex = -1
		this._deselectAll()
		this._select(row, col)
		this.anchor = row
		
		return
	}
	
	if(this.multiEnabled && ctrl && !shift) {
		this._toggle(row)
		this.anchor = row
	} else if(this.multiEnabled && shift && this.anchor != -1) {
		var dir = this.anchor < row ? +1 : -1
		var from = this.anchor + dir
		var to = row + dir
		if(this.isSelected(row)) {
			to -= dir
		}
		
		if(!ctrl) this._deselectExceptRange(from, to)
		if(!this.isSelected(row)) {
			for(var i = from; i != to; i += dir) {
				this._select(i, col)
			}
		}
	} else {
		if(this.editEnabled && this.anchor == row) {
			for(var i = 0; i < this._editLocks.length; i++) {
				if(this._editLocks[i] == row) return
			}
			this._startEdit(row, col)
			this._editIndex = row
			this._anchor = row
		} else {
			this._deselectAll()
			this._select(row, col)
			this.anchor = row
		}
	}
}
MultiSelection.prototype.setMultiSelection = function(enabled) {
	this.multiEnabled = enabled
}
MultiSelection.prototype.setEdit = function(enabled) {
	this.editEnabled = enabled
}

// Privates
MultiSelection.prototype._select = function(row, col) {
	this.selections.push(row)
	this.fireOnSelect(row, col)
}
MultiSelection.prototype._deselectAt = function(indexOfIndex) {
	var index = this.selections[indexOfIndex]
	this.selections.splice(indexOfIndex, 1)
	this.fireOnDeselect(index)
}
MultiSelection.prototype._deselectExceptRange = function(from, to) {
	for(var i = this.selections.length - 1; i >= 0; i--) {
		if((from > this.selections[i] || this.selections[i] > to) && this.selections[i] != this.anchor) this._deselectAt(i)
	}
}
MultiSelection.prototype._startEdit = function(row, col) {
	this.fireOnStartEdit(row, col)
}
MultiSelection.prototype._endEdit = function(row) {
	return this.fireOnEndEdit(row)
}

MultiSelection.prototype._toggle = function(row, col) {
	var indexOfIndex = -1
	for(var i = 0; i < this.selections.length; i++) {
		if(this.selections[i] == row) {
			indexOfIndex = i
			break
		}
	}
	if(indexOfIndex == -1) {
		this._select(row, col)
	} else {
		this._deselectAt(indexOfIndex)
	}
}
MultiSelection.prototype._deselectAll = function() {
	for(var i = 0; i < this.selections.length; i++) {
		this.fireOnDeselect(this.selections[i])
	}
	this.selections = new Array()
}

// Events
MultiSelection.prototype.addListener = function(l) {
	this.listeners.push(l)
}
MultiSelection.prototype.fireOnSelect = function(row, col) {
	for(var i = 0; i < this.listeners.length; i++) {
		this.listeners[i].onSelect(row, col)
	}
}
MultiSelection.prototype.fireOnDeselect = function(row) {
	for(var i = 0; i < this.listeners.length; i++) {
		this.listeners[i].onDeselect(row)
	}
}
MultiSelection.prototype.fireOnStartEdit = function(row, col) {
	for(var i = 0; i < this.listeners.length; i++) {
		this.listeners[i].onStartEdit(row, col)
	}
}
MultiSelection.prototype.fireOnEndEdit = function(row) {
	var canceled = false
	for(var i = 0; i < this.listeners.length; i++) {
		if(this.listeners[i].onEndEdit(row)) {
			canceled = true
		}
	}
	return canceled
}
MultiSelection.prototype.fireOnPreClick = function(row, col, ctrl, shift) {
	var cancel = false
	for(var i = 0; i < this.listeners.length; i++) {
		var temp = this.listeners[i].onPreClick(row, col, ctrl, shift)
		if(temp) {
			cancel = true
		}
	}
	return cancel
}